# install.packages(c("EpiModel", "extrafont", "animation"), 
#                  dependencies = TRUE)
library(EpiModel)
library(extrafont)
library(animation)


#
# Trying out a model.
#

param <- param.dcm(inf.prob = 0.2, act.rate = 1, rec.rate = 1/20,
                   a.rate = 0, ds.rate = 0, di.rate = 1/100, dr.rate = 0)
init <- init.dcm(s.num = 1000, i.num = 1, r.num = 0)
control <- control.dcm(type = "SIR", nsteps = 500, dt = 0.5)
mod <- dcm(param, init, control)
mod.df <- as.data.frame(mod)

# Plot it.
# par(family="Consolas", cex.axis=.9, las=1)
plot(mod.df$time, mod.df$i.num, type="l")



#
# Varying act.rate (Average number of transmissible acts per person per unit time)
#

init <- init.dcm(s.num = 1000, i.num = 1, r.num = 0)
control <- control.dcm(type = "SIR", nsteps = 1000, dt = 0.5)

# Customized plot.
par(family="Consolas", cex.axis=.9, las=1)
plot(NA, type="n", xlim=c(0,1000), ylim=c(0, 300),
     bty="n", axes=FALSE, 
     xlab="Time Since First Case", ylab="Number Infected",  
     main="Flatten the Curve")
axis(1, seq(0,1000,250), lwd=0, lwd.ticks = .5, pos = -5)
axis(2, at=seq(0, 250, 50), lwd=0, lwd.ticks=.5, pos=-2)

# Run simulations and draw lines.
act.rates <- seq(.8, 0, by=-.05)
for (rt in act.rates) {
    
    param <- param.dcm(inf.prob = 0.2, act.rate = rt, rec.rate = 1/20,
                       a.rate = 0, ds.rate = 0, di.rate = 1/100, dr.rate = 0)
    mod <- dcm(param, init, control)
    mod.df <- as.data.frame(mod)
    lines(mod.df$time, mod.df$i.num, 
          col=rgb(85/255,158/255,161/255,min(1-rt+.1, 1)),
          lwd=1+(1-rt))    
}



#
# Animate the curve.
#


saveGIF({
    
    # Baseline sim.
    param <- param.dcm(inf.prob = 0.2, act.rate = .8, rec.rate = 1/20,
                       a.rate = 0, ds.rate = 0, di.rate = 1/100, dr.rate = 0)
    mod.orig <- dcm(param, init, control)
    mod.orig.df <- as.data.frame(mod.orig)
    
    act.rates <- seq(.8, .1, by=-.02)
    for (rt in act.rates) {
        
        # Blank plot.
        par(family="Consolas", cex.axis=1.1, las=1, cex.main=2)
        plot(NA, type="n", xlim=c(0,1000), ylim=c(0, 300),
             bty="n", axes=FALSE, 
             xlab="Time Since First Case", ylab="Number Infected",  
             main="FLATTEN THE CURVE")
        mtext("WITH SOCIAL DISTANCE", side=3)
        axis(1, seq(0,1000,250), lwd=0, lwd.ticks = .5, pos = -5)
        axis(2, at=seq(0, 250, 50), lwd=0, lwd.ticks=.5, pos=-2)
        
        # Capacity marker
        lines(x=c(0,1000), y=c(100,100), lty=2)
        text(x=500, y=100, "Health Care Capacity", pos=3)
        
        # Line for baseline sim.
        lines(mod.orig.df$time, mod.orig.df$i.num,
              col=rgb(85/255,158/255,161/255,.5),
              lwd=1)
        
        # Simulate and draw current line.
        param <- param.dcm(inf.prob = 0.2, act.rate = rt, rec.rate = 1/20,
                           a.rate = 0, ds.rate = 0, di.rate = 1/100, dr.rate = 0)
        mod <- dcm(param, init, control)
        mod.df <- as.data.frame(mod)
        if (max(mod.df$i.num) <= 100) {
            linecol <- "#815C93"
            lwd <- 6
        } else {
            linecol <- rgb(85/255,158/255,161/255,1)
            lwd=3
        }
        
        lines(mod.df$time, mod.df$i.num, 
              col=linecol,
              lwd=lwd)    
    }
}, movie.name="flatten-curve-animated-w-capacity.gif", interval=.15, ani.width=670, ani.height=500)

